@@ -14,8 +14,8 @@ from hansken.abstract_trace import AbstractTrace
 from hansken.query import Term, to_query
 from hansken.util import (b64decode, b64encode,
                           DictView, flatten_mapping,
-                          format_datetime, GeographicLocation, parse_datetime,
-                          to_attr_name, Vector, view_with_attrs)
+                          format_datetime, GeographicLocation, INVALID_DATE,
+                          parse_datetime, to_attr_name, Vector, view_with_attrs)
 
 
 log = Logger(__name__)
@@ -347,8 +347,7 @@ class Trace(AbstractTrace):
             # default to empty sequence
             self._audits = self.get('system.processed.audit') or ()
             # createdOn should always be set, but don't crash when it's not available
-            # TODO: lexicographic ordering only works when timestamps are in the same timezone (HANSKENPY-177)
-            self._audits = sorted(self._audits, key=lambda audit: audit.get('createdOn') or '')
+            self._audits = sorted(self._audits, key=lambda audit: audit.get('createdOn') or INVALID_DATE)
 
         return self._audits
 
@@ -1095,13 +1094,13 @@ class TraceTypeView(DictView):
     source dict.
     """
 
-    def __init__(self, source, name, category, origins=('system', 'user'), converters=None):
+    def __init__(self, source, category, name, origins=('system', 'user'), converters=None):
         """
         Create a new trace type view.
 
         :param source: trace source
-        :param name: name of the trace type
         :param category: category where the named type resides
+        :param name: name of the trace type
         :param origins: origins where the trace type could be sourced from,
             ordered by priority
         :param converters: mapping of property names to optional type
@@ -1109,8 +1108,8 @@ class TraceTypeView(DictView):
         """
         super().__init__(source)
 
-        self._name = name
         self._category = category
+        self._name = name
         self._origins = origins
 
         self._converters = converters or {}
@@ -1160,6 +1159,38 @@ class TraceTypeView(DictView):
         return '<{0.__class__.__module__}.{0.__class__.__name__} ({0._name})>'.format(self)
 
 
+class TraceletTypeView(DictView):
+    """
+    Utility class to wrap a tracelet value that takes type conversions into
+    account.
+    """
+
+    def __init__(self, source, category, name, converters=None):
+        """
+        Create a new tracelet type view.
+
+        :param source: tracelet source
+        :param category: category where the named type resides
+        :param name: name of the trace type
+        :param converters: mapping of property names to optional type
+            converter (a `callable`)
+        """
+        super().__init__(source)
+
+        self._category = category
+        self._name = name
+        self._converters = converters or {}
+
+    def get(self, key, default=None):
+        # no origins or path trickery to apply, _source is the tracelet itself
+        value = super().get(key, default=self._no_default)
+        if value is not self._no_default:
+            converter = self._converters.get(key)
+            return converter(value) if converter else value
+        else:
+            return default
+
+
 def trace_class_from_model(model):
     """
     Creates a Trace class definition from the provided model. Both intrinsic
@@ -1192,19 +1223,19 @@ def trace_class_from_model(model):
     # create a reverse lookup to find the category of a type
     types = {name: category for category, name in model_types.keys()}
 
-    # determine many and mapped trace types
-    many_types = set()
+    # determine plural and mapped trace types
+    plural_types = set()
     mapped_types = set()
     for category, type_name in model_types.keys():
         trace_type = model['origins']['categories'][category]['types'][type_name]
-        # many-type defined by isList: true or few/many cardinality
-        if trace_type.get('isList') or trace_type.get('cardinality') in ('few', 'many'):
-            many_types.add((category, type_name))
+        # plural-type defined by few/many cardinality or (legacy) isList: true
+        if trace_type.get('cardinality') in ('few', 'many') or trace_type.get('isList'):
+            plural_types.add((category, type_name))
         # map-typed defined by "keys" being non-empty
         if trace_type.get('keys'):
             mapped_types.add((category, type_name))
 
-    # create a trace type class for each trace type defined by the model using type:
+    # create a trace type class for each singular / simple or mapped trace type defined by the model using type:
     # - new type is named alike extracted.text_message (builtin type's module "abc" is prefixed in __repr__)
     # - new type inherits from
     #   - TraceTypeView, able to get() values from a trace's deeply nested source dict
@@ -1214,7 +1245,17 @@ def trace_class_from_model(model):
     type_classes = {(category, type_name): type('.'.join(map(to_attr_name, (category, type_name))),
                                                 (TraceTypeView, view_with_attrs(type_name, properties.keys())),
                                                 {})
-                    for (category, type_name), properties in model_types.items()}
+                    for (category, type_name), properties in model_types.items()
+                    if (category, type_name) not in plural_types}
+    # analogous to the singular trace types, create Python types for the plural trace types, only difference being the
+    # TraceletTypeView over the TraceTypeView, that doesn't insert multi-origin trickery into the get() method (as
+    # tracelet values don't support deep paths anyway, there's a list index in the way somewhere)
+    # these types don't get the full trace source as their source, but just the tracelet value (see get() below)
+    type_classes.update({(category, type_name): type('.'.join(map(to_attr_name, (category, type_name))),
+                                                     (TraceletTypeView, view_with_attrs(type_name, properties.keys())),
+                                                     {})
+                         for (category, type_name), properties in model_types.items()
+                         if (category, type_name) in plural_types})
 
     # NB: the order of super classes is significant here, Trace needs the context argument, intrinsic_view (an
     #     AttrView) won't like it, method resolution order will make Trace.__init__ call intrinsic_view.__init__
@@ -1247,18 +1288,18 @@ def trace_class_from_model(model):
                             type_attr({
                                 # include key name to type name to make TraceTypeView construct the right selection key
                                 key: type_class(source,
-                                                '.'.join((type_name, key)), category_name, origins,
+                                                category_name, '.'.join((type_name, key)), origins,
                                                 converters=model_types.get((category_name, type_name)))
                                 for key in keys
                             }))
-                elif (category_name, type_name) in many_types:
-                    # make Trace class deal with many types
+                elif (category_name, type_name) in plural_types:
+                    # plural types don't automatically get an attribute
                     pass
                 else:
                     setattr(self,
                             to_attr_name(type_name),
                             type_class(source,
-                                       type_name, category_name, origins,
+                                       category_name, type_name, origins,
                                        converters=model_types.get((category_name, type_name))))
 
             # assign a set of type names to self
@@ -1300,6 +1341,11 @@ def trace_class_from_model(model):
                             if converter:
                                 # converter defined for value, apply
                                 return converter(value)
+                        if modeled and len(steps) == 2 and (steps[0], steps[1]) in plural_types:
+                            # explicitly getting a plural type (on-trace / few), wrap with a view that does not take
+                            # origins into account, accessing the local dict directly
+                            type_class = type_classes[(steps[0], steps[1])]
+                            return [type_class(element, steps[0], steps[1], modeled) for element in value]
 
                         return value
 
